<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin to configure the destination for a migration.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:configure_migration_destination',
  function_name: 'ai_migrate_agent_configure_migration_destination',
  name: 'Configure Migration Destination',
  description: 'Configures the destination plugin and settings for a Migration entity.',
  group: 'migrate_tools',
  context_definitions: [
    'migration_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Migration ID"),
      description: new TranslatableMarkup("The migration entity to configure."),
      required: TRUE
    ),
    'destination_plugin' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Destination Plugin"),
      description: new TranslatableMarkup("The destination plugin (e.g., entity:node, entity:user, etc.)."),
      required: TRUE
    ),
    'destination_configuration' => new ContextDefinition(
      data_type: 'any',
      label: new TranslatableMarkup("Destination Configuration"),
      description: new TranslatableMarkup("The configuration array for the destination plugin."),
      required: TRUE
    ),
  ]
)]
class ConfigureMigrationDestination extends FunctionCallBase implements ExecutableFunctionCallInterface {
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  public function execute() {
    $migration_id = $this->getContextValue('migration_id');
    $destination_plugin = $this->getContextValue('destination_plugin');
    $destination_configuration = $this->getContextValue('destination_configuration');
    $migration = \Drupal\migrate_plus\Entity\Migration::load($migration_id);
    if ($migration) {
      $destination = $migration->get('destination');
      $destination['plugin'] = $destination_plugin;
      $destination += $destination_configuration;
      $migration->set('destination', $destination);
      $migration->save();
      $this->setOutput('Destination configuration updated for migration "' . $migration_id . '".');
    } else {
      $this->setOutput('Migration entity not found: ' . $migration_id);
    }
  }
}
