<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin to configure the source for a migration.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:configure_migration_source',
  function_name: 'ai_migrate_agent_configure_migration_source',
  name: 'Configure Migration Source',
  description: 'Configures the source plugin and settings for a migration.',
  group: 'migrate_tools',
  context_definitions: [
    'migration_id' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Migration ID"),
      description: new TranslatableMarkup("The migration entity to configure."),
      required: TRUE
    ),
    'source_plugin' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Source Plugin"),
      description: new TranslatableMarkup("The source plugin (e.g., csv, json, url, etc.)."),
      required: TRUE
    ),
    'source_configuration' => new ContextDefinition(
      data_type: 'any',
      label: new TranslatableMarkup("Source Configuration"),
      description: new TranslatableMarkup("The configuration array for the source plugin."),
      required: TRUE
    ),
  ]
)]
class ConfigureMigrationSource extends FunctionCallBase implements ExecutableFunctionCallInterface {
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  public function execute() {
    $migration_id = $this->getContextValue('migration_id');
    $source_plugin = $this->getContextValue('source_plugin');
    $source_configuration = $this->getContextValue('source_configuration');
    $migration = \Drupal\migrate_plus\Entity\Migration::load($migration_id);
    if ($migration) {
      $source = $migration->get('source');
      $source['plugin'] = $source_plugin;
      $source += $source_configuration;
      $migration->set('source', $source);
      $migration->save();
      $this->setOutput('Source configuration updated for migration "' . $migration_id . '".');
    } else {
      $this->setOutput('Migration entity not found: ' . $migration_id);
    }
  }
}
