<?php

namespace Drupal\ai_migrate_agent\Plugin\AiFunctionCall;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Plugin\Context\ContextDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai\Attribute\FunctionCall;
use Drupal\ai\Base\FunctionCallBase;
use Drupal\ai\Service\FunctionCalling\ExecutableFunctionCallInterface;
use Drupal\ai\Service\FunctionCalling\FunctionCallInterface;
use Drupal\ai\Utility\ContextDefinitionNormalizer;
use Drupal\ai_agents\PluginInterfaces\AiAgentContextInterface;
use League\HTMLToMarkdown\HtmlConverter;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\migrate_plus\Entity\MigrationGroup;

/**
 * Plugin implementation of the field validation results.
 */
#[FunctionCall(
  id: 'ai_migrate_agent:create_migration_group',
  function_name: 'ai_migrate_agent_create_migration_group',
  name: 'Create Migration Group',
  description: 'Create a Migration Group entity for organizing migrations using the Migrate module.',
  group: 'migrate_tools',
  context_definitions: [
    'name' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Name"),
      description: new TranslatableMarkup("The human-readable name of the Migration Group to make"),
      required: TRUE
    ),
    'description' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Description"),
      description: new TranslatableMarkup("An optional description of the Migration Group."),
      required: FALSE
    ),
    'source_type' => new ContextDefinition(
      data_type: 'string',
      label: new TranslatableMarkup("Source Type"),
      description: new TranslatableMarkup("The type of source for the migration (CSV, Table, URL, etc)."),
      required: TRUE
    ),
  ]
)]
class CreateMigrationGroup extends FunctionCallBase implements ExecutableFunctionCallInterface, AiAgentContextInterface {

  /**
   * Load from dependency injection container.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FunctionCallInterface|static {
    $instance = new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      new ContextDefinitionNormalizer(),
    );
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function execute() {
    // Collect the context values.
    $name = $this->getContextValue('name');

    if (empty($name)) {
      $this->setOutput('Failed to create Migration Group because the name was empty.');
      return;
    }

    $id = trim(strtolower(str_replace(' ', '_', trim($name))));
    if (empty($id)) {
      $this->setOutput('Failed to create Migration Group because the ID was empty.');
      return;
    }
    $description = $this->getContextValue('description');
    $source_type = $this->getContextValue('source_type');

    if (empty($source_type)) {
      $this->setOutput('Failed to create Migration Group because a Source Type is required.');
      return;
    }

    // Clean up the source type.
    $source_type = strtolower(str_replace(' ', '_', $source_type));

    $options = [
      'label' => $name,
      'id' => $id,
      'source_type' => $source_type,
    ];

    if (!empty($description)) {
      $options['description'] = $description;
    }

    // Create a Migration Group entity.
    $migration_group = MigrationGroup::create($options);

    // Save the entity.
    if ($migration_group->save()) {
      $this->setOutput('The Migration Group was created.');
    }
    else {
      $this->setOutput('There was an error creating the Migration Group.');
    }
  }

}
